@@ -311,7 +311,7 @@ class log_monitor_manager {
     }
     
     private function check_ftp_logs() {
-        $log_file = $this->get_config('ftp_log_file', '/var/log/proftpd/auth.log');
+        $log_file = $this->get_config('ftp_log_file', '/var/log/proftpd/access.log');
         
         // Validate log file path to prevent path traversal
         if (!$this->is_safe_log_path($log_file)) {
@@ -485,50 +485,35 @@ class log_monitor_manager {
             return;
         }
         
-        // ProFTPD access.log format for failed logins
-        // Format can be either:
-        // 1. IP_ADDRESS username - [timestamp] "PASS (hidden)" 530 -
-        // 2. hostname username - [timestamp] "PASS (hidden)" 530 -
-        // Status code 530 means authentication failed
-        // access.log typically has more complete logging than auth.log
+        // ProFTPD access.log format (IP-only format after config fix):
+        // Failed auth: IP_ADDRESS UNKNOWN - [timestamp] "PASS (hidden)" 530 -
+        // Failed REIN: IP_ADDRESS UNKNOWN - [timestamp] "REIN" 502 -
+        // Status codes: 530 = authentication failed, 502 = command error (potential scanning)
         
-        // First try to extract IP from the first field (could be IP or hostname)
-        if (preg_match('/^(\S+)\s+(\S+)\s+-\s+\[.*?\]\s+"PASS \(hidden\)" 530/', $line, $matches)) {
-            $first_field = $matches[1];
-            $username = $matches[2];
+        // Pattern to match failed login attempts (530) and excessive REIN errors (502)
+        if (preg_match('/^(\d+\.\d+\.\d+\.\d+)\s+(\S+)\s+-\s+\[.*?\]\s+"(?:PASS \(hidden\)|REIN)"\s+(530|502)/', $line, $matches)) {
+            $ip = $matches[1];
+            $username_field = $matches[2];
+            $status_code = $matches[3];
             
-            // Check if first field is an IP address
-            if (filter_var($first_field, FILTER_VALIDATE_IP)) {
-                $ip = $first_field;
-            } else {
-                // Try to extract IP from hostname patterns
-                // Common patterns: 123.45.67.89.bc.googleusercontent.com, ec2-123-45-67-89.compute-1.amazonaws.com
-                if (preg_match('/(\d+[\.\-]\d+[\.\-]\d+[\.\-]\d+)/', $first_field, $ip_matches)) {
-                    // Replace dashes with dots for EC2-style hostnames
-                    $ip = str_replace('-', '.', $ip_matches[1]);
-                    // Validate the extracted IP
-                    if (!filter_var($ip, FILTER_VALIDATE_IP)) {
-                        return false; // Can't extract valid IP
-                    }
-                } else {
-                    // Can't extract IP from hostname, skip this entry
-                    return false;
-                }
-            }
-            
-            $username = ($username !== '-' && $username !== 'UNKNOWN') ? $username : 'unknown';
-            
-            // Validate extracted data
+            // Validate IP address
             if (filter_var($ip, FILTER_VALIDATE_IP)) {
+                // Clean up username - if it's UNKNOWN, use 'unknown'
+                $username = ($username_field !== 'UNKNOWN' && $username_field !== '-') ? $username_field : 'unknown';
+                
                 // Sanitize username (allow alphanumeric, underscore, dash, max 32 chars)
                 $username = substr(preg_replace('/[^a-zA-Z0-9_-]/', '', $username), 0, 32);
                 if (empty($username)) {
                     $username = 'unknown';
                 }
                 
+                // Record the failed attempt
                 $this->record_service_failed_attempt('ftp', $ip, $username, $line);
+                return true;
             }
         }
+        
+        return false;
     }
     
     private function record_service_failed_attempt($service, $ip, $username, $log_line) {
