@@ -12,13 +12,59 @@ class security_manager {
         $this->init_database();
     }
     
+    /**
+     * Execute a database query with retry logic for handling locks
+     * 
+     * @param callable $operation The database operation to execute
+     * @param int $max_retries Maximum number of retry attempts
+     * @param int $base_delay Base delay in microseconds between retries
+     * @return mixed Result of the operation, or false on failure
+     */
+    private function execute_with_retry($operation, $max_retries = 5, $base_delay = 100000) {
+        for ($attempt = 1; $attempt <= $max_retries; $attempt++) {
+            try {
+                $result = $operation();
+                
+                // If operation succeeded, return the result
+                if ($result !== false) {
+                    return $result;
+                }
+                
+                // If this is not the last attempt, wait and retry
+                if ($attempt < $max_retries) {
+                    usleep($base_delay * $attempt); // Progressive delay
+                    continue;
+                }
+                
+                // All retries failed
+                error_log("Security Manager: Database operation failed after $max_retries attempts");
+                return false;
+                
+            } catch (Exception $e) {
+                error_log("Security Manager: Database operation error (attempt $attempt/$max_retries): " . $e->getMessage());
+                
+                // If this is not the last attempt, wait and retry
+                if ($attempt < $max_retries) {
+                    usleep($base_delay * $attempt); // Progressive delay
+                    continue;
+                }
+                
+                // All retries failed
+                return false;
+            }
+        }
+        
+        return false;
+    }
+    
     private function init_database() {
         if (!file_exists($this->db_path)) {
             $this->db = new SQLite3($this->db_path);
-            $this->db->busyTimeout(6000);
+            $this->db->busyTimeout(10000); // Increase timeout to 10 seconds
+            $this->db->exec('PRAGMA journal_mode = WAL'); // Use Write-Ahead Logging for better concurrency
             
             // Create failed login attempts table
-            $this->db->exec('CREATE TABLE failed_logins (
+            $this->db->exec('CREATE TABLE IF NOT EXISTS failed_logins (
                 id INTEGER PRIMARY KEY AUTOINCREMENT,
                 ip_address TEXT NOT NULL,
                 username TEXT NOT NULL,
@@ -27,7 +73,7 @@ class security_manager {
             )');
             
             // Create blocked IPs table
-            $this->db->exec('CREATE TABLE blocked_ips (
+            $this->db->exec('CREATE TABLE IF NOT EXISTS blocked_ips (
                 id INTEGER PRIMARY KEY AUTOINCREMENT,
                 ip_address TEXT UNIQUE NOT NULL,
                 block_time INTEGER NOT NULL,
@@ -38,7 +84,7 @@ class security_manager {
             )');
             
             // Create whitelist IPs table
-            $this->db->exec('CREATE TABLE whitelist_ips (
+            $this->db->exec('CREATE TABLE IF NOT EXISTS whitelist_ips (
                 id INTEGER PRIMARY KEY AUTOINCREMENT,
                 ip_address TEXT UNIQUE NOT NULL,
                 description TEXT,
@@ -46,7 +92,7 @@ class security_manager {
             )');
             
             // Create successful logins table
-            $this->db->exec('CREATE TABLE successful_logins (
+            $this->db->exec('CREATE TABLE IF NOT EXISTS successful_logins (
                 id INTEGER PRIMARY KEY AUTOINCREMENT,
                 username TEXT NOT NULL,
                 ip_address TEXT NOT NULL,
@@ -55,14 +101,15 @@ class security_manager {
             )');
             
             // Create indexes for better performance
-            $this->db->exec('CREATE INDEX idx_failed_logins_ip ON failed_logins(ip_address)');
-            $this->db->exec('CREATE INDEX idx_failed_logins_time ON failed_logins(attempt_time)');
-            $this->db->exec('CREATE INDEX idx_blocked_ips_time ON blocked_ips(unblock_time)');
-            $this->db->exec('CREATE INDEX idx_successful_logins_user ON successful_logins(username)');
-            $this->db->exec('CREATE INDEX idx_successful_logins_time ON successful_logins(login_time)');
+            $this->db->exec('CREATE INDEX IF NOT EXISTS idx_failed_logins_ip ON failed_logins(ip_address)');
+            $this->db->exec('CREATE INDEX IF NOT EXISTS idx_failed_logins_time ON failed_logins(attempt_time)');
+            $this->db->exec('CREATE INDEX IF NOT EXISTS idx_blocked_ips_time ON blocked_ips(unblock_time)');
+            $this->db->exec('CREATE INDEX IF NOT EXISTS idx_successful_logins_user ON successful_logins(username)');
+            $this->db->exec('CREATE INDEX IF NOT EXISTS idx_successful_logins_time ON successful_logins(login_time)');
         } else {
             $this->db = new SQLite3($this->db_path);
-            $this->db->busyTimeout(6000);
+            $this->db->busyTimeout(10000); // Increase timeout to 10 seconds
+            $this->db->exec('PRAGMA journal_mode = WAL'); // Use Write-Ahead Logging for better concurrency
             
             // Check if successful_logins table exists, create if not (for existing databases)
             $this->ensure_successful_logins_table();
@@ -70,20 +117,59 @@ class security_manager {
     }
     
     private function ensure_successful_logins_table() {
-        // Check if successful_logins table exists
-        $result = $this->db->query("SELECT name FROM sqlite_master WHERE type='table' AND name='successful_logins'");
-        if (!$result->fetchArray()) {
-            // Create the table and indexes
-            $this->db->exec('CREATE TABLE successful_logins (
-                id INTEGER PRIMARY KEY AUTOINCREMENT,
-                username TEXT NOT NULL,
-                ip_address TEXT NOT NULL,
-                login_time INTEGER NOT NULL,
-                user_agent TEXT
-            )');
-            
-            $this->db->exec('CREATE INDEX idx_successful_logins_user ON successful_logins(username)');
-            $this->db->exec('CREATE INDEX idx_successful_logins_time ON successful_logins(login_time)');
+        // Check if successful_logins table exists with retry logic
+        $max_retries = 5;
+        $retry_delay = 100000; // 100ms in microseconds
+        
+        for ($attempt = 1; $attempt <= $max_retries; $attempt++) {
+            try {
+                $result = $this->db->query("SELECT name FROM sqlite_master WHERE type='table' AND name='successful_logins'");
+                
+                // Check if query was successful
+                if ($result === false) {
+                    // If this is not the last attempt, wait and retry
+                    if ($attempt < $max_retries) {
+                        usleep($retry_delay * $attempt); // Progressive delay
+                        continue;
+                    }
+                    // If all retries failed, log error and return
+                    error_log("Security Manager: Failed to query sqlite_master after $max_retries attempts");
+                    return;
+                }
+                
+                // Check if table exists
+                if (!$result->fetchArray()) {
+                    // Create the table and indexes
+                    $this->db->exec('CREATE TABLE IF NOT EXISTS successful_logins (
+                        id INTEGER PRIMARY KEY AUTOINCREMENT,
+                        username TEXT NOT NULL,
+                        ip_address TEXT NOT NULL,
+                        login_time INTEGER NOT NULL,
+                        user_agent TEXT
+                    )');
+                    
+                    // Create indexes if they don't exist
+                    $this->db->exec('CREATE INDEX IF NOT EXISTS idx_successful_logins_user ON successful_logins(username)');
+                    $this->db->exec('CREATE INDEX IF NOT EXISTS idx_successful_logins_time ON successful_logins(login_time)');
+                }
+                
+                // Success - exit the retry loop
+                return;
+                
+            } catch (Exception $e) {
+                // Log the error
+                error_log("Security Manager: Database operation failed (attempt $attempt/$max_retries): " . $e->getMessage());
+                
+                // If this is not the last attempt, wait and retry
+                if ($attempt < $max_retries) {
+                    usleep($retry_delay * $attempt); // Progressive delay
+                    continue;
+                }
+                
+                // If all retries failed, log final error
+                error_log("Security Manager: Failed to ensure successful_logins table after $max_retries attempts");
+                return;
+            }
         }
     }
     
@@ -358,39 +444,51 @@ class security_manager {
     public function get_last_logins($username) {
         $this->ensure_successful_logins_table();
         
-        $stmt = $this->db->prepare('SELECT * FROM successful_logins WHERE username = :user ORDER BY login_time DESC LIMIT 3');
-        if (!$stmt) {
-            return array(); // Return empty array if table doesn't exist
-        }
-        
-        $stmt->bindValue(':user', $username, SQLITE3_TEXT);
-        $result = $stmt->execute();
-        
-        $logins = array();
-        while ($row = $result->fetchArray(SQLITE3_ASSOC)) {
-            $logins[] = $row;
-        }
-        
-        return $logins;
+        return $this->execute_with_retry(function() use ($username) {
+            $stmt = $this->db->prepare('SELECT * FROM successful_logins WHERE username = :user ORDER BY login_time DESC LIMIT 3');
+            if (!$stmt) {
+                return array(); // Return empty array if table doesn't exist
+            }
+            
+            $stmt->bindValue(':user', $username, SQLITE3_TEXT);
+            $result = $stmt->execute();
+            
+            if ($result === false) {
+                return false; // Trigger retry
+            }
+            
+            $logins = array();
+            while ($row = $result->fetchArray(SQLITE3_ASSOC)) {
+                $logins[] = $row;
+            }
+            
+            return $logins;
+        }) ?: array(); // Return empty array on complete failure
     }
     
     public function get_all_recent_logins($limit = 10) {
         $this->ensure_successful_logins_table();
         
-        $stmt = $this->db->prepare('SELECT * FROM successful_logins ORDER BY login_time DESC LIMIT :limit');
-        if (!$stmt) {
-            return array(); // Return empty array if table doesn't exist
-        }
-        
-        $stmt->bindValue(':limit', $limit, SQLITE3_INTEGER);
-        $result = $stmt->execute();
-        
-        $logins = array();
-        while ($row = $result->fetchArray(SQLITE3_ASSOC)) {
-            $logins[] = $row;
-        }
-        
-        return $logins;
+        return $this->execute_with_retry(function() use ($limit) {
+            $stmt = $this->db->prepare('SELECT * FROM successful_logins ORDER BY login_time DESC LIMIT :limit');
+            if (!$stmt) {
+                return array(); // Return empty array if table doesn't exist
+            }
+            
+            $stmt->bindValue(':limit', $limit, SQLITE3_INTEGER);
+            $result = $stmt->execute();
+            
+            if ($result === false) {
+                return false; // Trigger retry
+            }
+            
+            $logins = array();
+            while ($row = $result->fetchArray(SQLITE3_ASSOC)) {
+                $logins[] = $row;
+            }
+            
+            return $logins;
+        }) ?: array(); // Return empty array on complete failure
     }
     
     public function get_security_stats() {
